/**
 ** Written and arranged by Yoshito Hirata (yoshito@sat.t.u-tokyo.ac.jp)
 ** This program was prepared for Bioinformatics programming contest 2005.
 ** This is an implimentation for finding the set of parameters that
 ** explains the given data.
 ** The key features are multiple shooting and differential evolution.
 ** By using multiple shooting, we can use a cost function that is easy
 ** to minimize. Differential evolution speeds up the minimization.
 */

/***************************************************************
**                                                            **
**        D I F F E R E N T I A L     E V O L U T I O N       **
**                                                            **
** Program: de.c                                              **
** Version: 3.6                                               **
**                                                            **
** Authors: Dr. Rainer Storn                                  **
**          c/o ICSI, 1947 Center Street, Suite 600           **
**          Berkeley, CA 94707                                **
**          Tel.:   510-642-4274 (extension 192)              **
**          Fax.:   510-643-7684                              **
**          E-mail: storn@icsi.berkeley.edu                   **
**          WWW: http://http.icsi.berkeley.edu/~storn/        **
**          on leave from                                     **
**          Siemens AG, ZFE T SN 2, Otto-Hahn Ring 6          **
**          D-81739 Muenchen, Germany                         **
**          Tel:    636-40502                                 **
**          Fax:    636-44577                                 **
**          E-mail: rainer.storn@zfe.siemens.de               **
**                                                            **
**          Kenneth Price                                     **
**          836 Owl Circle                                    **
**          Vacaville, CA 95687                               **
**          E-mail: kprice@solano.community.net               ** 
**                                                            **
** This program implements some variants of Differential      **
** Evolution (DE) as described in part in the techreport      **
** tr-95-012.ps of ICSI. You can get this report either via   **
** ftp.icsi.berkeley.edu/pub/techreports/1995/tr-95-012.ps.Z  **
** or via WWW: http://http.icsi.berkeley.edu/~storn/litera.html*
** A more extended version of tr-95-012.ps is submitted for   **
** publication in the Journal Evolutionary Computation.       ** 
**                                                            **
** You may use this program for any purpose, give it to any   **
** person or change it according to your needs as long as you **
** are referring to Rainer Storn and Ken Price as the origi-  **
** nators of the the DE idea.                                 **
** If you have questions concerning DE feel free to contact   **
** us. We also will be happy to know about your experiences   **
** with DE and your suggestions of improvement.               **
**                                                            **
***************************************************************/
/**H*O*C**************************************************************
**                                                                  **
** No.!Version! Date ! Request !    Modification           ! Author **
** ---+-------+------+---------+---------------------------+------- **
**  1 + 3.1  +5/18/95+   -     + strategy DE/rand-to-best/1+  Storn **
**    +      +       +         + included                  +        **
**  1 + 3.2  +6/06/95+C.Fleiner+ change loops into memcpy  +  Storn **
**  2 + 3.2  +6/06/95+   -     + update comments           +  Storn **
**  1 + 3.3  +6/15/95+ K.Price + strategy DE/best/2 incl.  +  Storn **
**  2 + 3.3  +6/16/95+   -     + comments and beautifying  +  Storn **
**  3 + 3.3  +7/13/95+   -     + upper and lower bound for +  Storn **
**    +      +       +         + initialization            +        **
**  1 + 3.4  +2/12/96+   -     + increased printout prec.  +  Storn **
**  1 + 3.5  +5/28/96+   -     + strategies revisited      +  Storn **
**  2 + 3.5  +5/28/96+   -     + strategy DE/rand/2 incl.  +  Storn **
**  1 + 3.6  +8/06/96+ K.Price + Binomial Crossover added  +  Storn **
**  2 + 3.6  +9/30/96+ K.Price + cost variance output      +  Storn **
**  3 + 3.6  +9/30/96+   -     + alternative to ASSIGND    +  Storn **
**  4 + 3.6  +10/1/96+   -    + variable checking inserted +  Storn **
**  5 + 3.6  +10/1/96+   -     + strategy indic. improved  +  Storn **
**                                                                  **
***H*O*C*E***********************************************************/

/*#include "de.h"*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#define UT_DEBUG2
#define UT_DEBUG3
#define UT_DEBUG6
#define PENALTY_POSITIVE
#define PENALTY_MODERATE
#define PENALTY_UNSTABLE
/*#define PENALTY_POSITIVE2*/
#define PENALTY 10000
#define MAXPOP  500
#define MAXDIM  52
#define MU (0.1)

/*------Constants for rnd_uni()--------------------------------------------*/

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

#define VTR (10)
#define STEPSIZE (0.01)
/*#define NP (10)*/
#define ITERMAX (3000)
/*#define F (0.7)*/
/*#define CR (0.8)*/
#define XVMIN (0)
#define XVMAX (1)
#define REFLESH (10)
#define BIGNUM (-10000000)
#define TH (5)
#define NE (10)
#define NE2 (10)
#define TINY (0.000001)
#define MAX_NSV (200)

#define NPARAMS (21)
#define NDIMS (3)

#define D (NDIMS)
#define P (NPARAMS)

/*#define th1 (2)*/
/*#define th2 (13)*/

#define KK (10)
#define GAP (0.5)
#define GAP2 (0.1)
#define GAP3 (0.5)


/*------------------------Globals---------------------------------------*/

long  rnd_uni_init;                 /* serves as a seed for rnd_uni()   */
/*double c[MAXPOP][MAXDIM], d[MAXPOP][MAXDIM];*/
/*double (*pold)[MAXPOP][MAXDIM], (*pnew)[MAXPOP][MAXDIM], (*pswap)[MAXPOP][MAXDIM];*/

/*---------Function declarations----------------------------------------*/

double h(double x);
void  assignd(int DD, double a[], double b[]);
double rnd_uni(long *idum);    /* uniform pseudo random number generator */
/*double evaluate(int D, double tmp[], long *nfeval);*/ /* obj. funct. */
double **file_read(char filename[],int *cc, int *strategy);
double **file_read2(char filename[],int *cc, int *strategy, int *cc2);
double A(double xx[NDIMS],double *params,double t,double nx[NDIMS],double u);
double A2(double xx[NDIMS],double *params,double t,double nx[NDIMS],double u);
double cost_function2(double **data,int cc,int DD,double *params,long *nfeval,double **sim);
void orig_quick_sort(int *v, int *seq, int left, int right);
void randperm(int *ind, int *seq, int cc);
void find_params(double **data, double params[NPARAMS], int cc);
int ut(double *xhat,double **Pxx,double *y,double **R,double t,double **xsigma,double u);
/*void choldc(double **a, int n, double p[]);*/

/*---------Function definitions-----------------------------------------*/

/*void choldc(double **a, int n, double p[])
{
	int i,j,k;
	double sum;

	for (i=0;i<n;i++) {
		for (j=i;j<n;j++) {
			for (sum=a[i][j],k=i-1;k>=0;k--) sum -= a[i][k]*a[j][k];
			if (i == j) {
				if (sum <= 0.0)
					fprintf(stderr,"choldc failed, sum:%lf\n",sum);
				p[i]=sqrt(sum);
			} else a[j][i]=sum/p[i];
		}
	}
}
*/

double **file_read2(char filename[],int *cc, int *strategy, int *cc2){
  double **data;
  FILE *fp,*fp0;
  char dummy[100];
  char buffer[100];
  double u,a,x1,x2,y,t;
  int i,j,k,l;
  int cycles;

  data = NULL;

  if((fp = fopen(filename,"r")) == NULL){
    fprintf(stderr,"file: %s cannot be opened.\n",filename);
    return data; 
  }
  fprintf(stderr,"opened file.\n");

  /*  fgets(dummy,100,fp);*/
  fprintf(stderr,"fgeted: %s.\n",dummy);

  *cc = 0;

  while(fgets(dummy,100,fp) > 0){
    /*    fprintf(stderr,"%s\n",dummy);*/
    (*cc)++;
  }

  fprintf(stderr,"scanned file once: cc = %i\n",*cc);

  if((data = (double **)malloc(sizeof(double *)*(*cc))) == NULL){
    fprintf(stderr,"cannot allocate memory for data.\n");
    return data;
  } 

  fprintf(stderr,"malloced memory for data 1.\n");
  
  for(i=0;i<(*cc);i++){
    if((data[i] = (double *)malloc(sizeof(double)*4)) == NULL){
      fprintf(stderr,"cannot allocate memory for data.\n");
      return data;
    }
  }

  fprintf(stderr,"malloced memory for data 2.\n");


  fclose(fp);

  if((fp = fopen(filename,"r")) == NULL){
    fprintf(stderr,"file: %s cannot be opened.\n",filename);
    return data; 
  }

  i = 0;

  /*fgets(dummy,100,fp);*/
  while(fgets(dummy,100,fp) > 0){

    /*find a patient number.*/
    j = 0; k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*    if(j == 0){
      t = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        t = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&t);
      }
    }*/

    /*find date*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*
    if(k == 0){
      g = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        g = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&g);
      }
    }
    */

    /*find the number of days for medication*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*
    if(k == 0){
      h = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        h = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&h);
      }
    }
    */

    /*find the number of days for a cycle*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*
    if(k == 0){
      p = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        p = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&p);
      }
    }
    */

    /*find psa*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      y = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        y = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&y);
      }
    }


    /*find testosterone*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      a = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        a = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&a);
      }
    }


    /*find cycle*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*    if(k == 0){
      a = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        a = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&a);
      }
      }*/


    /*find medication*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      u = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        u = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&u);
      }
    }


    /*find the number of days in total*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      t = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        t = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&t);
      }
    }


    j++;  k = 0;
    while((dummy[j] != '\0')&&(dummy[j] != '\n')&&(dummy[j] != 13)){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*    printf("i=%i,k=%i:",i,k);
    for(l=0;l<k;l++){
      printf("%c:%i,",buffer[l],buffer[l]);
    }
    printf(",");
    if(k == 0){
      z = BIGNUM;
    }else{
      if((buffer[k-2] == ' ')||(buffer[k-1] == '\0')||(buffer[k-1] == '\n')||(buffer[k-1] == 13)){
        z = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&z);
      }
    }
    printf("z=%f\n",z);
    */

    data[i][0] = y;
    data[i][1] = u;
    data[i][2] = t;
    i++;
  }

  *cc2 = *cc;
  cycles = 0;
  for(i=1;i<*cc;i++){
    if((data[i-1][1] == 1)&&(data[i][1] == 0)){
      cycles++;
      if(cycles == 2){
	*cc2 = i+1;
      }
    }
  }


  fprintf(stderr,"scanned the file second time\n");
  fprintf(stderr,"cc2:%i\n",*cc2);


  fclose(fp);
  fprintf(stderr,"closed file.\n");

  return data;
}

double h(double x){
  double temp = 0;

  if(x < 0){
    temp = 1-x;
  }

  return temp;
}




void  assignd(int DD, double a[], double b[])
/**C*F****************************************************************
**                                                                  **
** Assigns D-dimensional vector b to vector a.                      **
** You might encounter problems with the macro ASSIGND on some      **
** machines. If yes, better use this function although it's slower. **
**                                                                  **
***C*F*E*************************************************************/
{
   int j;
   for (j=0; j<DD; j++)
   {
      a[j] = b[j];
   }
}


double rnd_uni(long *idum)
/**C*F****************************************************************
**                                                                  **
** SRC-FUNCTION   :rnd_uni()                                        **
** LONG_NAME      :random_uniform                                   **
** AUTHOR         :(see below)                                      **
**                                                                  **
** DESCRIPTION    :rnd_uni() generates an equally distributed ran-  **
**                 dom number in the interval [0,1]. For further    **
**                 reference see Press, W.H. et alii, Numerical     **
**                 Recipes in C, Cambridge University Press, 1992.  **
**                                                                  **
** FUNCTIONS      :none                                             **
**                                                                  **
** GLOBALS        :none                                             **
**                                                                  **
** PARAMETERS     :*idum    serves as a seed value                  **
**                                                                  **
** PRECONDITIONS  :*idum must be negative on the first call.        **
**                                                                  **
** POSTCONDITIONS :*idum will be changed                            **
**                                                                  **
***C*F*E*************************************************************/
{
  long j;
  long k;
  static long idum2=123456789;
  static long iy=0;
  static long iv[NTAB];
  double temp;

  if (*idum <= 0)
  {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    idum2=(*idum);
    for (j=NTAB+7;j>=0;j--)
    {
      k=(*idum)/IQ1;
      *idum=IA1*(*idum-k*IQ1)-k*IR1;
      if (*idum < 0) *idum += IM1;
      if (j < NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ1;
  *idum=IA1*(*idum-k*IQ1)-k*IR1;
  if (*idum < 0) *idum += IM1;
  k=idum2/IQ2;
  idum2=IA2*(idum2-k*IQ2)-k*IR2;
  if (idum2 < 0) idum2 += IM2;
  j=iy/NDIV;
  iy=iv[j]-idum2;
  iv[j] = *idum;
  if (iy < 1) iy += IMM1;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;

}/*------End of rnd_uni()--------------------------*/



double **file_read(char filename[],int *cc, int *strategy){
  double **data;
  FILE *fp,*fp0;
  char dummy[100];
  char buffer[100];
  double u,a,x1,x2,y,t,p;
  int i,j,k,l;

  data = NULL;

  if((fp = fopen(filename,"r")) == NULL){
    fprintf(stderr,"file: %s cannot be opened.\n",filename);
    return data; 
  }
  fprintf(stderr,"opened file.\n");

  /*  fgets(dummy,100,fp);*/
  fprintf(stderr,"fgeted: %s.\n",dummy);

  *cc = 0;

  while(fgets(dummy,100,fp) > 0){
    /*    fprintf(stderr,"%s\n",dummy);*/
    (*cc)++;
  }

  fprintf(stderr,"scanned file once: cc = %i\n",*cc);

  if((data = (double **)malloc(sizeof(double *)*(*cc))) == NULL){
    fprintf(stderr,"cannot allocate memory for data.\n");
    return data;
  } 

  fprintf(stderr,"malloced memory for data 1.\n");
  
  for(i=0;i<(*cc);i++){
    if((data[i] = (double *)malloc(sizeof(double)*3)) == NULL){
      fprintf(stderr,"cannot allocate memory for data.\n");
      return data;
    }
  }

  fprintf(stderr,"malloced memory for data 2.\n");


  fclose(fp);

  if((fp = fopen(filename,"r")) == NULL){
    fprintf(stderr,"file: %s cannot be opened.\n",filename);
    return data; 
  }

  i = 0;

  /*  fgets(dummy,100,fp);*/
  while(fgets(dummy,100,fp) > 0){

    /*find psa.*/
    j = 0; k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(j == 0){
      y = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        y = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&y);
      }
    }

    /*find the number of cycle*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    /*    if(k == 0){
      g = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        g = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&g);
      }
      }*/

    /*find the medication procedure*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      u = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        u = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&u);
      }
    }


    /*find the number of days*/
    j++;  k = 0;
    while(dummy[j] != ','){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      t = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        t = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&t);
      }
    }

    /*find the patient number*/
    j++;  k = 0;
    while(dummy[j] != '\n'){
      buffer[k] = dummy[j];
      j++; k++;
    }
    if(k == 0){
      p = BIGNUM;
    }else{
      if(buffer[k-1] == ' '){
        p = BIGNUM;
      }else{
	buffer[k] = ',';
	sscanf(buffer,"%lg,",&p);
      }
    }

    fprintf(stderr,"y=%f,u=%f,t=%f,p=%f\n",y,u,t,p);
    data[i][0] = y;
    data[i][1] = u;
    data[i][2] = t;
    data[i][3] = p;
    i++;
  }

  fprintf(stderr,"scanned the file second time\n");


  fclose(fp);
  fprintf(stderr,"closed file.\n");

  return data;
}


double A(double xx[NDIMS],double *params,double t,double nx[NDIMS],double u){
  int i,j,k,n;
  double temp[NDIMS],temp2;
  int temp3;

  for(i=0;i<D;i++){
    temp[i] = xx[i];
  }

  for(i=0;i<t;i++){
    /*    if(u == 2){
      for(j=0;j<NDIMS;j++){
	nx[j] = params[20+j]*temp[j]+params[24+j];
	temp[j] = nx[j];
      }
      }*/
    if(u == 1){
      A2(temp,params,0,nx,1);
      for(j=0;j<NDIMS;j++){
	temp[j] = nx[j];
      }
    }
    if(u == 0){
      A2(temp,params,0,nx,0);
      for(j=0;j<NDIMS;j++){
	temp[j] = nx[j];
      }
    }
  }

  return 0.0;
}

double A2(double xx[NDIMS],double *params,double t,double nx[NDIMS],double u){
  int i,j,k,n;
  double temp2;
  int temp3;

  /*    if(u == 2){
      for(j=0;j<NDIMS;j++){
	nx[j] = params[20+j]*xx[j]+params[24+j];
      }
      }*/
    if(u == 1){

      nx[0] = params[3]*xx[0]+params[13];
      nx[1] = params[4]*xx[0]+params[5]*xx[1]+params[14];
      nx[2] = params[6]*xx[0]+params[7]*xx[1]+params[8]*xx[2]+params[15];

    }
    if(u == 0){
      nx[0] = params[9]*xx[0]+params[10]*xx[1]+params[16];
      nx[1] = params[11]*xx[1]+params[17];
      nx[2] = params[12]*xx[2]+params[18];
    }


  return 0.0;
}


double cost_function2(double **data,int cc,int DD,double *params, long *nfeval,double **sim){
  double cost = 0.001;
  int i,j,k,p=0,m,pm,ppm,delays = 0;
  double xx[NDIMS];
  double xin[NDIMS];
  double temp,temp2;
  double cost2 = 0;
  int count = cc;
  int nill = 0;
  int cill = 0;
  double meanxxx[13];
  double pinvcovxxx[13][13];
  double mhalfdetcovxxx;
  double prev_model_psa,prev_actual_psa;
  /*  double c1,c2;*/

  /*  c1 = params[6];
      c2 = params[7];*/

  (*nfeval)++;

  meanxxx[0] = 16.71938583;
  meanxxx[1] = 0.8337136504;
  meanxxx[2] = 0.08582438261;
  meanxxx[3] = 0.9202517222;
  meanxxx[4] = 0.008896737443;
  meanxxx[5] = 0.9975550556;
  meanxxx[6] = 0.0001447412197;
  meanxxx[7] = 0.0001355197058;
  meanxxx[8] = 1.009078333;
  meanxxx[9] = 1.002198444;
  meanxxx[10] = 0.08106036472;
  meanxxx[11] = 1.000527361;
  meanxxx[12] = 0.956075;

    pinvcovxxx[0][0] = 0.02109604437;
    pinvcovxxx[0][1] = 0.0207156396;
    pinvcovxxx[0][2] = -0.912686128;
    pinvcovxxx[0][3] = -0.8216801105;
    pinvcovxxx[0][4] = 2.226065364;
    pinvcovxxx[0][5] = 2.974153814;
    pinvcovxxx[0][6] = -17.72296086;
    pinvcovxxx[0][7] = 152.2713512;
    pinvcovxxx[0][8] = -3.978238498;
    pinvcovxxx[0][9] = -5.871130099;
    pinvcovxxx[0][10] = 1.758432938;
    pinvcovxxx[0][11] = 2.040786801;
    pinvcovxxx[0][12] = -0.6177861032;

    pinvcovxxx[1][0] = 0.0207156396;
    pinvcovxxx[1][1] = 0.7108189307;
    pinvcovxxx[1][2] = -2.053032954;
    pinvcovxxx[1][3] = -1.681089372;
    pinvcovxxx[1][4] = 19.46442608;
    pinvcovxxx[1][5] = 181.3208974;
    pinvcovxxx[1][6] = -880.6424913;
    pinvcovxxx[1][7] = 764.8369361;
    pinvcovxxx[1][8] = -19.52398345;
    pinvcovxxx[1][9] = -86.53606405;
    pinvcovxxx[1][10] = 3.219352951;
    pinvcovxxx[1][11] = 78.83771889;
    pinvcovxxx[1][12] = -2.605823499;

    pinvcovxxx[2][0] = -0.912686128;
    pinvcovxxx[2][1] = -2.053032954;
    pinvcovxxx[2][2] = 83.02791138;
    pinvcovxxx[2][3] = 52.48923306;
    pinvcovxxx[2][4] = -78.0373153;
    pinvcovxxx[2][5] = -387.2373124;
    pinvcovxxx[2][6] = -912.2828496;
    pinvcovxxx[2][7] = -21783.36195;
    pinvcovxxx[2][8] = 472.483913;
    pinvcovxxx[2][9] = 973.0639601;
    pinvcovxxx[2][10] = -66.66623797;
    pinvcovxxx[2][11] = -647.4910934;
    pinvcovxxx[2][12] = 84.57168972;

    pinvcovxxx[3][0] = -0.8216801105;
    pinvcovxxx[3][1] = -1.681089372;
    pinvcovxxx[3][2] = 52.48923306;
    pinvcovxxx[3][3] = 687.8302179;
    pinvcovxxx[3][4] = 1088.245856;
    pinvcovxxx[3][5] = 1061.339472;
    pinvcovxxx[3][6] = 7636.080135;
    pinvcovxxx[3][7] = -20047.96501;
    pinvcovxxx[3][8] = 198.001461;
    pinvcovxxx[3][9] = 572.3663906;
    pinvcovxxx[3][10] = -125.7161487;
    pinvcovxxx[3][11] = -149.7329391;
    pinvcovxxx[3][12] = 18.23884593;

    pinvcovxxx[4][0] = 2.226065364;
    pinvcovxxx[4][1] = 19.46442608;
    pinvcovxxx[4][2] = -78.0373153;
    pinvcovxxx[4][3] = 1088.245856;
    pinvcovxxx[4][4] = 7282.845005;
    pinvcovxxx[4][5] = 11134.57007;
    pinvcovxxx[4][6] = -30847.107;
    pinvcovxxx[4][7] = 5051.873209;
    pinvcovxxx[4][8] = -992.3832574;
    pinvcovxxx[4][9] = -351.4315649;
    pinvcovxxx[4][10] = 1594.176372;
    pinvcovxxx[4][11] = 3971.539441;
    pinvcovxxx[4][12] = -215.9637116;

    pinvcovxxx[5][0] = 2.974153814;
    pinvcovxxx[5][1] = 181.3208974;
    pinvcovxxx[5][2] = -387.2373124;
    pinvcovxxx[5][3] = 1061.339472;
    pinvcovxxx[5][4] = 11134.57007;
    pinvcovxxx[5][5] = 71993.23238;
    pinvcovxxx[5][6] = -331333.2162;
    pinvcovxxx[5][7] = 142207.079;
    pinvcovxxx[5][8] = -6841.036621;
    pinvcovxxx[5][9] = -25727.14722;
    pinvcovxxx[5][10] = 1324.740822;
    pinvcovxxx[5][11] = 31258.58534;
    pinvcovxxx[5][12] = -595.4156746;

    pinvcovxxx[6][0] = -17.72296086;
    pinvcovxxx[6][1] = -880.6424913;
    pinvcovxxx[6][2] = -912.2828496;
    pinvcovxxx[6][3] = 7636.080135;
    pinvcovxxx[6][4] = -30847.107;
    pinvcovxxx[6][5] = -331333.2162;
    pinvcovxxx[6][6] = 10096611.55;
    pinvcovxxx[6][7] = -168879.3382;
    pinvcovxxx[6][8] = 52113.74868;
    pinvcovxxx[6][9] = -90123.02073;
    pinvcovxxx[6][10] = 2380.379441;
    pinvcovxxx[6][11] = 171468.9003;
    pinvcovxxx[6][12] = 5913.357343;

    pinvcovxxx[7][0] = 152.2713512;
    pinvcovxxx[7][1] = 764.8369361;
    pinvcovxxx[7][2] = -21783.36195;
    pinvcovxxx[7][3] = -20047.96501;
    pinvcovxxx[7][4] = 5051.873209;
    pinvcovxxx[7][5] = 142207.079;
    pinvcovxxx[7][6] = -168879.3382;
    pinvcovxxx[7][7] = 10081844.06;
    pinvcovxxx[7][8] = -116678.7256;
    pinvcovxxx[7][9] = -292124.8131;
    pinvcovxxx[7][10] = 516.5971832;
    pinvcovxxx[7][11] = 187574.8048;
    pinvcovxxx[7][12] = -27751.28434;

    pinvcovxxx[8][0] = -3.978238498;
    pinvcovxxx[8][1] = -19.52398345;
    pinvcovxxx[8][2] = 472.483913;
    pinvcovxxx[8][3] = 198.001461;
    pinvcovxxx[8][4] = -992.3832574;
    pinvcovxxx[8][5] = -6841.036621;
    pinvcovxxx[8][6] = 52113.74868;
    pinvcovxxx[8][7] = -116678.7256;
    pinvcovxxx[8][8] = 11520.5328;
    pinvcovxxx[8][9] = 9692.763504;
    pinvcovxxx[8][10] = -237.1705541;
    pinvcovxxx[8][11] = -6235.609698;
    pinvcovxxx[8][12] = 748.1109209;

    pinvcovxxx[9][0] = -5.871130099;
    pinvcovxxx[9][1] = -86.53606405;
    pinvcovxxx[9][2] = 973.0639601;
    pinvcovxxx[9][3] = 572.3663906;
    pinvcovxxx[9][4] = -351.4315649;
    pinvcovxxx[9][5] = -25727.14722;
    pinvcovxxx[9][6] = -90123.02073;
    pinvcovxxx[9][7] = -292124.8131;
    pinvcovxxx[9][8] = 9692.763504;
    pinvcovxxx[9][9] = 46376.94667;
    pinvcovxxx[9][10] = 332.4101549;
    pinvcovxxx[9][11] = -27154.39613;
    pinvcovxxx[9][12] = 802.2221596;

    pinvcovxxx[10][0] = 1.758432938;
    pinvcovxxx[10][1] = 3.219352951;
    pinvcovxxx[10][2] = -66.66623797;
    pinvcovxxx[10][3] = -125.7161487;
    pinvcovxxx[10][4] = 1594.176372;
    pinvcovxxx[10][5] = 1324.740822;
    pinvcovxxx[10][6] = 2380.379441;
    pinvcovxxx[10][7] = 516.5971832;
    pinvcovxxx[10][8] = -237.1705541;
    pinvcovxxx[10][9] = 332.4101549;
    pinvcovxxx[10][10] = 1876.870587;
    pinvcovxxx[10][11] = 1473.749449;
    pinvcovxxx[10][12] = 55.60649907;

    pinvcovxxx[11][0] = 2.040786801;
    pinvcovxxx[11][1] = 78.83771889;
    pinvcovxxx[11][2] = -647.4910934;
    pinvcovxxx[11][3] = -149.7329391;
    pinvcovxxx[11][4] = 3971.539441;
    pinvcovxxx[11][5] = 31258.58534;
    pinvcovxxx[11][6] = 171468.9003;
    pinvcovxxx[11][7] = 187574.8048;
    pinvcovxxx[11][8] = -6235.609698;
    pinvcovxxx[11][9] = -27154.39613;
    pinvcovxxx[11][10] = 1473.749449;
    pinvcovxxx[11][11] = 39067.38752;
    pinvcovxxx[11][12] = -859.9806712;

    pinvcovxxx[12][0] = -0.6177861032;
    pinvcovxxx[12][1] = -2.605823499;
    pinvcovxxx[12][2] = 84.57168972;
    pinvcovxxx[12][3] = 18.23884593;
    pinvcovxxx[12][4] = -215.9637116;
    pinvcovxxx[12][5] = -595.4156746;
    pinvcovxxx[12][6] = 5913.357343;
    pinvcovxxx[12][7] = -27751.28434;
    pinvcovxxx[12][8] = 748.1109209;
    pinvcovxxx[12][9] = 802.2221596;
    pinvcovxxx[12][10] = 55.60649907;
    pinvcovxxx[12][11] = -859.9806712;
    pinvcovxxx[12][12] = 291.1039408;




  mhalfdetcovxxx = 47596384920.0*10000000000.0;


  temp = 0;
  for(j=0;j<NDIMS;j++){
    xin[j] = params[j];
    xx[j] = params[j];
    sim[0][j] = xin[j];
    temp += xin[j];
    cost2 += PENALTY*h(sim[0][j]);
  }
  sim[0][NDIMS] = temp;
  if(data[0][0] == BIGNUM){
    count--;
  }else{
    temp = (sim[0][NDIMS]-data[0][0]);
    /*if(temp < 0){
      temp = -temp;
      }*/
    cost += temp*temp;
  /*  fprintf(stderr,"%f\n",temp);*/
    prev_actual_psa = data[0][0];
    prev_model_psa = sim[0][NDIMS];
  }


  ppm = 1-data[0][1];
  pm = data[0][1];
  m = data[0][1];
  delays = 0;
  for(i=1;i<cc;i++){
   if(pm != m){
     delays = 0;
     ppm = pm;
   }
   if(m == 0){
     for(j=0;j<data[i][2]-data[i-1][2];j++){
       delays--;
       if(delays < -params[19]){
	 A2(xin,params,1,xx,0);
       }else{
	 A2(xin,params,1,xx,1);
       }
       for(k=0;k<NDIMS;k++){
	 xin[k] = xx[k];
       }
     }
   }
   if(m == 1){
     for(j=0;j<data[i][2]-data[i-1][2];j++){
       delays--;
	/*	if(ppm == 0){*/
       if(delays < -params[20]){
	 A2(xin,params,1,xx,1);
       }else{
	 A2(xin,params,1,xx,0);
       }
       for(k=0;k<NDIMS;k++){
	 xin[k] = xx[k];
       }
     } 
   }


   temp = 0;
   for(j=0;j<NDIMS;j++){
     xin[j] = xx[j];
     sim[i][j] = xin[j];
     temp += xin[j];
     cost2 += PENALTY*h(sim[i][j]);
   }
   sim[i][NDIMS] = temp;


   if(data[i][0] == BIGNUM){
     count--;
   }else{
     temp = (sim[i][NDIMS]-data[i][0]);
     /*if(temp < 0){
       temp = -temp;
       }*/
     /*     fprintf(stderr,"%f\n",temp);*/
     cost += temp*temp;

     prev_actual_psa = data[i][0];
     prev_model_psa = sim[i][NDIMS];
   }
   pm = m;
   m = data[i][1];
  }
  /*cost /= (count+0.000000001);*/
  cost += cost2;

  temp = 0;
  for(j=0;j<13;j++){
      for(k=0;k<13;k++){
	temp += (params[j]-meanxxx[j])*(pinvcovxxx[j][k])*(params[k]-meanxxx[k]);
      }
  }

  temp2 = 1;
  for(j=0;j<13;j++){
    temp2 = temp2*sqrt(2*3.141592653);
  }

  temp = temp/2 + log(temp2*mhalfdetcovxxx);

  cost += temp;

  for(j=0;j<NDIMS;j++){
    xin[j] = params[j];
  }
  A(xin,params,30*12,xx,1);
  temp = 0;
  for(j=0;j<NDIMS;j++){
    temp += xx[j];
    xin[j] = xx[j];
  }

  cost += PENALTY*h(2-temp);
  A(xin,params,30*12*4,xx,1);
  temp = 0;
  for(j=0;j<NDIMS;j++){
    temp += xx[j];
  }
  cost += PENALTY*h(temp-10);

  /*  return cost+10*(params[9]<0)+10*(params[10]<0);*/
#ifdef PENALTY_POSITIVE
  for(j=0;j<P;j++){
    cost += PENALTY*h(params[j]);
  }
#endif
#ifdef PENALTY_MODERATE

  cost += PENALTY*h(params[3]-0.8);
  cost += PENALTY*h(1.2-params[3]);
  cost += PENALTY*h(params[5]-0.8);
  cost += PENALTY*h(1.2-params[5]);
  cost += PENALTY*h(params[8]-0.8);
  cost += PENALTY*h(1.2-params[8]);

  cost += PENALTY*h(params[9]-0.8);
  cost += PENALTY*h(1.2-params[9]);
  cost += PENALTY*h(params[11]-0.8);
  cost += PENALTY*h(1.2-params[11]);
  cost += PENALTY*h(params[12]-0.8);
  cost += PENALTY*h(1.2-params[12]);
  /*  for(j=20;j<24;j++){
    cost += PENALTY*h(params[j]-0.8);
    cost += PENALTY*h(1.2-params[j]);
    }*/


  cost += PENALTY*h(params[4]-0.0);
  cost += PENALTY*h(0.1-params[4]);
  cost += PENALTY*h(params[6]-0.0);
  cost += PENALTY*h(0.1-params[6]);
  cost += PENALTY*h(params[7]-0.0);
  cost += PENALTY*h(0.1-params[7]);
  cost += PENALTY*h(params[10]-0.0);
  cost += PENALTY*h(0.1-params[10]);


  cost += PENALTY*h(params[13]-0.0);
  cost += PENALTY*h(0.1-params[13]);
  cost += PENALTY*h(params[14]-0.0);
  cost += PENALTY*h(0.1-params[14]);
  cost += PENALTY*h(params[15]-0.0);
  cost += PENALTY*h(0.1-params[15]);

  cost += PENALTY*h(params[16]-0.0);
  cost += PENALTY*h(0.1-params[16]);
  cost += PENALTY*h(params[17]-0.0);
  cost += PENALTY*h(0.1-params[17]);
  cost += PENALTY*h(params[18]-0.0);
  cost += PENALTY*h(0.1-params[18]);

  /*Ideas from Prof. Tao*/
  cost += PENALTY*h(params[9]-params[3]);
  cost += PENALTY*h(params[5]-params[3]);
  cost += PENALTY*h(params[9]-1);


  /*  for(j=32;j<40;j++){
    cost += PENALTY*h(params[j]-0.0);
    cost += PENALTY*h(0.1-params[j]);
    }*/
  /*  for(j=24;j<28;j++){
    cost += PENALTY*h(params[j]-0.0);
    cost += PENALTY*h(0.2-params[j]);
    }*/
#endif
#ifdef PENALTY_UNSTABLE
  temp = 0;
  temp += params[3];
  temp += params[4];
  temp += params[6];

  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);

  temp = 0;
  temp += params[5];
  temp += params[7];

  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);

  temp = 0;
  temp += params[8];
  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);


  temp = 0;
  temp += params[9];
  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);

  temp = 0;
  temp += params[10];
  temp += params[11];
  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);


  temp = 0;
  temp += params[12];
  cost += PENALTY*h(temp-0.8);
  cost += PENALTY*h(1.2-temp);



  /* cost += PENALTY*h(params[12]-1);*/
  cost += PENALTY*h(params[8]-1);
  /*  cost += PENALTY*h(1-params[4]);
      cost += PENALTY*h(1-params[8]);*/
#endif
  return cost;
}



main(int argc, char *argv[])
/**C*F****************************************************************
**                                                                  **
** SRC-FUNCTION   :main()                                           **
** LONG_NAME      :main program                                     **
** AUTHOR         :Rainer Storn, Kenneth Price                      **
**                                                                  **
** DESCRIPTION    :driver program for differential evolution.       **
**                                                                  **
** FUNCTIONS      :rnd_uni(), evaluate(), printf(), fprintf(),      **
**                 fopen(), fclose(), fscanf().                     **
**                                                                  **
** GLOBALS        :rnd_uni_init    input variable for rnd_uni()     **
**                                                                  **
** PARAMETERS     :argc            #arguments = 3                   **
**                 argv            pointer to argument strings      **
**                                                                  **
** PRECONDITIONS  :main must be called with three parameters        **
**                 e.g. like de1 <input-file> <output-file>, if     **
**                 the executable file is called de1.               **
**                 The input file must contain valid inputs accor-  **
**                 ding to the fscanf() section of main().          **
**                                                                  **
** POSTCONDITIONS :main() produces consecutive console outputs and  **
**                 writes the final results in an output file if    **
**                 the program terminates without an error.         **
**                                                                  **
***C*F*E*************************************************************/

{
   char  chr;             /* y/n choice variable                */
   int   b, bs, i, j, k, l, L, n, ii;      /* counting variables                 */
   int   r1, r2;  /* placeholders for random indexes    */
   int   r5;              /* placeholders for random indexes    */
   int   NP;              /* number of population members       */
   int   imin;            /* index to member with lowest energy */
   int   refresh;         /* refresh rate of screen output      */
   int   refresh2 = 1;         /* refresh rate of screen output      */
   int   strategy = 1;        /* choice parameter for screen output */
   int   gen, genmax, seed;   

   long  nfeval;          /* number of function evaluations     */

   double trial_cost;      /* buffer variable                    */
   double inibound_h;      /* upper parameter bound              */
   double inibound_l;      /* lower parameter bound              */
   double *tmp, *best, *best2, *best3, *bestit; /* members  */
   double bestsofar;
   double cost[MAXPOP];    /* obj. funct. values                 */
   double cvar;            /* computes the cost variance         */
   double cmean;           /* mean cost                          */
   double F,CR;            /* control variables of DE            */
   double cmin, minti;            /* help variables                     */
   double **data;
   double **sim;
   int cc, cc2 = 100;
   int quick_check;

   FILE  *fpin_ptr, *fp;

   double prev_cmin = 0;
   int nsv = 0;
   int c_quick = 0;
   int seq[100];
   int dd = 0;

   double xhat[D+P];

   double temp,temp2;
   double var[D];
   int count;
   double **R;
   double Q[D][D];
   double **Pxx;

   double **xsigma;
   int ut_count;
   double **c, **d;
   double ***pold, ***pnew, ***pswap;

   double xin[3];
   double xx[3];
   double u;
   double th1 = 15, th2 = 2;

   int DP;

   bs = 50;
   bestsofar = 1000000;

   DP = D + P;

/*------Initializations----------------------------*/

 if (argc != 2)                                 /* number of arguments */
 {
    printf("\nUsage : bip2005 <data-file>\n");
    exit(1);
 }


/*-----Read input data------------------------------------------------*/

 genmax = 10000;
 refresh = 10;
 NP = (P)*10;
 inibound_h = 1;
 inibound_l = -1;
 F = 0.7;
 CR = 0.8;
 seed = 11;


/*-----Checking input variables for proper range----------------------------*/

  if (D > MAXDIM)
  {
     printf("\nError! D=%d > MAXDIM=%d\n",D,MAXDIM);
     exit(1);
  }
  if (D <= 0)
  {
     printf("\nError! D=%d, should be > 0\n",D);
     exit(1);
  }
  if (NP > MAXPOP)
  {
     printf("\nError! NP=%d > MAXPOP=%d\n",NP,MAXPOP);
     exit(1);
  }
  if (NP <= 0)
  {
     printf("\nError! NP=%d, should be > 0\n",NP);
     exit(1);
  }
  if ((CR < 0) || (CR > 1.0))
  {
     printf("\nError! CR=%lf, should be ex [0,1]\n",CR);
     exit(1);
  }
  if (seed <= 0)
  {
     printf("\nError! seed=%d, should be > 0\n",seed);
     exit(1);
  }
  if (refresh <= 0)
  {
     printf("\nError! refresh=%d, should be > 0\n",refresh);
     exit(1);
  }
  if (genmax <= 0)
  {
     printf("\nError! genmax=%d, should be > 0\n",genmax);
     exit(1);
  }
  if (inibound_h < inibound_l)
  {
     printf("\nError! inibound_h=%lf < inibound_l=%lf\n",inibound_h, inibound_l);
     exit(1);
  }


/*-----read data------------------------------------------------------*/
  data = file_read2(argv[1],&cc,&strategy,&cc2);
  strategy = 8;
  /*  cc2 = cc;*/
  /*  sscanf(argv[2],"%i\n",&cc2);
  sscanf(argv[3],"%lf\n",&th1);
  sscanf(argv[4],"%lf\n",&th2);
  printf("cc2=%i, th1=%lf, th2=%lf\n",cc2,th1,th2);*/

 fpin_ptr   = fopen("input.dat","w");

 fprintf(fpin_ptr,"TIME,PSA,MED\n");
 i = 0;
 fprintf(fpin_ptr,"%.3f,%.2E,%.2E\n",data[i][2],data[i][0],data[i][1]);
 for(i=1;i<cc;i++){
   fprintf(fpin_ptr,"%.3f,%.2E,%.2E\n",data[i][2],data[i][0],data[i][1]);
 }

 fclose(fpin_ptr);

 if((c = (double **)malloc(sizeof(double *)*NP)) == NULL){
   exit(1);
 }

 for(i=0;i<NP;i++){
   if((c[i] = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
     exit(1);
   }
 }

 if((d = (double **)malloc(sizeof(double *)*NP)) == NULL){
   exit(1);
 }

 for(i=0;i<NP;i++){
   if((d[i] = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
     exit(1);
   }
 }

 if((tmp = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
   exit(1);
 }

 if((best = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
   exit(1);
 }

 if((best2 = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
   exit(1);
 }

 if((best3 = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
   exit(1);
 }

 if((
bestit = (double *)malloc(sizeof(double)*(NPARAMS+2*cc))) == NULL){
   exit(1);
 }



 if((sim = (double **)malloc(sizeof(double *)*(cc))) == NULL){
   exit(1);
 }
 for(i=0;i<cc;i++){
   if((sim[i] = (double *)malloc(sizeof(double)*(NDIMS+1))) == NULL){
     exit(1);
   }
 }


  /*
 dd = cc;

  for(i=0;(i<100)&&(dd>0);i++){
    seq[i] = dd;
    dd = dd/2;
  }
  */

/*-----Initialize random number generator-----------------------------*/

 for(b=0;b<bs;b++){

  quick_check = 0;

  seed = (int)time(0);
  printf("seed=%i\n",seed);

  rnd_uni_init = -(long)seed;  /* initialization of rnd_uni() */
  nfeval       =  0;  /* reset number of function evaluations */

  fprintf(stderr,"multiple shooting with short integral interval, seed:%i\n",seed);


/*------Initialization------------------------------------------------*/
/*------Right now this part is kept fairly simple and just generates--*/
/*------random numbers in the range [-initfac, +initfac]. You might---*/
/*------want to extend the init part such that you can initialize-----*/
/*------each parameter separately.------------------------------------*/

  fprintf(stderr,"initialization\n");
  for (i=0; i<NP; i++)
    {
      /*      fprintf(stderr,"i=%i\n",i);*/
      
      inibound_l = 15;
      inibound_h = 5;
      c[i][0] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 1;
      inibound_h = 0;
      c[i][1] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 1;
      inibound_h = 0;
      c[i][2] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][3] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.2;
      inibound_h = 0.0;
      c[i][4] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][5] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.2;
      inibound_h = 0.0;
      c[i][6] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][7] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][8] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][9] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][10] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][11] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 1.2;
      inibound_h = 0.8;
      c[i][12] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][13] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][14] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][15] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][16] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][17] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][18] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0;
      inibound_h = 0;
      c[i][19] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0;
      inibound_h = 0;
      c[i][20] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      /*
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][21] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][22] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0.0;
      c[i][23] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0;
      c[i][24] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.0;
      inibound_h = 0;
      c[i][25] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.0;
      inibound_h = 0;
      c[i][26] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0;
      c[i][27] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.0;
      inibound_h = 0;
      c[i][28] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0;
      inibound_h = 0;
      c[i][29] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0;
      inibound_h = 0;
      c[i][30] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      */
      /*
      inibound_l = 30;
      inibound_h = 0;
      c[i][31] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      */
      /*
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][32] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][33] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][34] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][35] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      inibound_l = 0.2;
      inibound_h = 0;
      c[i][36] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][37] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][38] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 0.2;
      inibound_h = 0;
      c[i][39] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      */
      /*
      inibound_l = 10;
      inibound_h = 0;
      c[i][32] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 10;
      inibound_h = 0;
      c[i][33] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      
      inibound_l = 10;
      inibound_h = 0;
      c[i][20] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
      inibound_l = 10;
      inibound_h = 0;
      c[i][21] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

      */
	  /*      cost[i] = evaluate(D,c[i],&nfeval);*/ /* obj. funct. value */
	  /*	cost[i] = cost_function(data,seq[dd],D,c[i],&nfeval);*/ /* obj. funct. value */
      cost[i] = cost_function2(data,cc2,D,c[i],&nfeval,sim);/* obj. funct. value */
      while(((cost[i] < 0) == 0)&&((cost[i] >= 0) == 0)){


	inibound_l = 15;
	inibound_h = 5;
	c[i][0] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1;
	inibound_h = 0;
	c[i][1] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1;
	inibound_h = 0;
	c[i][2] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][3] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 0.2;
	inibound_h = 0.0;
	c[i][4] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][5] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 0.2;
	inibound_h = 0.0;
	c[i][6] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][7] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][8] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][9] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0.0;
	c[i][10] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][11] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 1.2;
	inibound_h = 0.8;
	c[i][12] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][13] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][14] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][15] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][16] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][17] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][18] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0;
	inibound_h = 0;
	c[i][19] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0;
	inibound_h = 0;
	c[i][20] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	/*
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][21] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][22] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0.0;
	inibound_h = 0.0;
	c[i][23] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0;
	c[i][24] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0;
	c[i][25] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0.0;
	inibound_h = 0;
	c[i][26] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0;
	c[i][27] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.0;
	inibound_h = 0;
	c[i][28] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0;
	inibound_h = 0;
	c[i][29] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	
	inibound_l = 0;
	inibound_h = 0;
	c[i][30] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	*/
	/*
	inibound_l = 30;
	inibound_h = 0;
	c[i][31] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	*/
	/*
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][32] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][33] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][34] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][35] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

	inibound_l = 0.2;
	inibound_h = 0;
	c[i][36] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][37] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][38] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	inibound_l = 0.2;
	inibound_h = 0;
	c[i][39] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);
	*/

	  /*	  cost[i] = cost_function(data,seq[dd],D,c[i],&nfeval);*/ /* obj. funct. value */
	cost[i] = cost_function2(data,cc2,D,c[i],&nfeval,sim); /* obj. funct. value */
      }

      /*      for(j=0;j<cc2;j++){
	fprintf(stderr,"%lf,%lf,%lf,%lf,%lf\n",sim[j][0],sim[j][1],sim[j][2],sim[j][3],sim[j][4]);
      }
      fprintf(stderr,"cost[%i] = %lf\n",i,cost[i]);*/
    }
  cmin = cost[0];
  imin = 0;
  for (i=1; i<NP; i++)
    {
      if (cost[i]<cmin)
	{
	  cmin = cost[i];
	  imin = i;
	}
    }
    
  assignd((NPARAMS+2*cc),best,c[imin]);            /* save best member ever          */
  assignd((NPARAMS+2*cc),bestit,c[imin]);          /* save best member of generation */
      
  pold = &c; /* old population (generation G)   */
  pnew = &d; /* new population (generation G+1) */
      
/*=======================================================================*/
/*=========Iteration loop================================================*/
/*=======================================================================*/
    
  gen = 0;                          /* generation counter reset */

    /*    fprintf(stderr,"cmin:%lf\n",cmin);*/
  prev_cmin = cmin+10;

  quick_check = 0;
    /*   while ((gen < genmax) && ((cmin >= 1)||((cmin < 1)&&(quickly_check_goodness(data,best,cc2) == 0))))*/ /* remove comments if conio.h */
  fprintf(stderr,"start differential evolution.\n");

  while (gen < genmax){                                            /* is accepted by compiler    */
    gen++;
    imin = 0;
	  
    if(cmin == prev_cmin)nsv++;
    prev_cmin = cmin;
	  
    for (i=0; i<NP; i++)         /* Start of loop through ensemble  */
      {
	do                        /* Pick a random population member */
	  {                         /* Endless loop for NP < 2 !!!     */
	    r1 = (int)(rnd_uni(&rnd_uni_init)*NP);
	  }while(r1==i);            
	
	do                        /* Pick a random population member */
	  {                         /* Endless loop for NP < 3 !!!     */
	    r2 = (int)(rnd_uni(&rnd_uni_init)*NP);
	  }while((r2==i) || (r2==r1));
	  
	  
/*=======EXPONENTIAL CROSSOVER============================================================*/

/*-------DE/rand-to-best/1/bin-----------------------------------------------------------*/
	assignd((NPARAMS+2*cc),tmp,(*pold)[i]);
	n = (int)(rnd_uni(&rnd_uni_init)*(P)); 
	for (L=0; L<P; L++) /* perform D binomial trials */
	  {
	    if ((rnd_uni(&rnd_uni_init) < CR) || L == (P-1)) /* change at least one parameter */
	      {                       
		tmp[n] = tmp[n] + F*(bestit[n] - tmp[n]) + F*((*pold)[r1][n]-(*pold)[r2][n]);
	      }
	    n = (n+1)%P;
	  }
	  
/*=======Trial mutation now in tmp[]. Test how good this choice really was.==================*/

	 /*	 trial_cost = evaluate(D,tmp,&nfeval);*/  /* Evaluate new vector in tmp[] */
	    /*	    trial_cost = cost_function(data,seq[dd],D,tmp,&nfeval);*/  /* Evaluate new vector in tmp[] */
	trial_cost = cost_function2(data,cc2,D,tmp,&nfeval,sim);  /* Evaluate new vector in tmp[] */
	/*	if(i==0){
	  minti = trial_cost;
	  assignd((NPARAMS),bestit,tmp);
	}else{
	  if(trial_cost < minti){
	  minti = trial_cost;
	  assignd((NPARAMS),bestit,tmp);
	  }
	  }*/

	if (trial_cost <= cost[i])   /* improved objective function value ? */
	  {                                  
	    cost[i]=trial_cost;         
	    assignd((NPARAMS+2*cc),(*pnew)[i],tmp);
	    if (trial_cost<cmin)          /* Was this a new minimum? */
	      {                               /* if so...*/
		cmin=trial_cost;           /* reset cmin to new low...*/
		imin=i;
		assignd((NPARAMS+2*cc),best,tmp);           
	      }                           
	  }                            
	else
	  {
	    assignd((NPARAMS+2*cc),(*pnew)[i],(*pold)[i]); /* replace target with old value */
	  }
      }   /* End mutation loop through pop. */
	
   assignd((NPARAMS+2*cc),bestit,best); /* Save best population member of current iteration */

	/* swap population arrays. New generation becomes old one */
	
    pswap = pold;
    pold  = pnew;
    pnew  = pswap;

/*----Compute the energy variance (just for monitoring purposes)-----------*/

    cmean = 0.;          /* compute the mean value first */
    for (j=0; j<NP; j++)
      {
	cmean += cost[j];
      }
    cmean = cmean/NP;
    
    cvar = 0.;           /* now the variance              */
    for (j=0; j<NP; j++)
      {
	cvar += (cost[j] - cmean)*(cost[j] - cmean);
      }
    cvar = cvar/(NP-1);
	

/*----Output part----------------------------------------------------------*/

/* display after every refresh generations */
/* ABORT works only if conio.h is accepted by your compiler */
/*    if (gen%refresh==0)   
      { 
	printf("\n\n                         PRESS ANY KEY TO ABORT"); 
	printf("\n\n\n Best-so-far cost funct. value=%-15.10g\n",cmin);
	
	for (j=0;j<P;j++)
	  {
	    printf("\n best[%d]=%-15.10g",j,best[j]);
	  }
	printf("\n\n Generation=%d  NFEs=%ld   Strategy: %i    ",gen,nfeval,strategy);
	printf("\n NP=%d    F=%-4.2g    CR=%-4.2g   cost-variance=%-10.5g\n",
	       NP,F,CR,cvar);
	printf("\n cost_function2 with best: %lf\n",cost_function2(data,cc2,D,best,&nfeval,sim));
	printf("\n cost_function2 with bestit: %lf\n",cost_function2(data,cc2,D,bestit,&nfeval,sim));
      }
  }*/

  }

  if(bestsofar > cmin){
    /*    bestsofar = cmin;*/
    printf("\n cmin: %lf\n",cmin);

    printf("\n\n                         PRESS ANY KEY TO ABORT"); 
    printf("\n\n\n Best-so-far cost funct. value=%-15.10g\n",cmin);
	
    for (j=0;j<P;j++)
    {
        printf("\n best[%d]=%-15.10g",j,best[j]);
    }
    printf("\n\n Generation=%d  NFEs=%ld   Strategy: %i    ",gen,nfeval,strategy);
    printf("\n NP=%d    F=%-4.2g    CR=%-4.2g   cost-variance=%-10.5g\n",NP,F,CR,cvar);

    for(i=0;i<(NPARAMS+2*cc);i++){
      best3[i] = best[i];
    }
    for (j=0;j<P;j++)
    {
        printf("\n best3[%d]=%-15.10g",j,best3[j]);
    }
    printf("\n cost_function2 with best: %lf\n",cost_function2(data,cc2,D,best,&nfeval,sim));
    bestsofar = cost_function2(data,cc2,D,best,&nfeval,sim);
  }
 }


/*=======================================================================*/
/*=========End of iteration loop=========================================*/
/*=======================================================================*/



/*-------Final output in file-------------------------------------------*/




  fprintf(stderr,"evaluation:%lf\n",cmin);

   /*   fpin_ptr   = fopen("temp.dat","w");

   fprintf(fpin_ptr,"TIME,G,H,P,X,Y,Z\n");
   j = 0;
   fprintf(fpin_ptr,"%.3f,%.2E,%.2E,%.2E,%.2E,%.2E,%.2E\n",sim[j][0],sim[j][1],sim[j][2],sim[j][3],sim[j][4],sim[j][5],sim[j][6]);
   for(j=1;j<cc2;j++){
     fprintf(fpin_ptr,"%.3f,%.2E,%.2E,%.2E,%.2E,%.2E,%.2E\n",sim[j][0],sim[j][1],sim[j][2],sim[j][3],sim[j][4],sim[j][5],sim[j][6]);
   }

   fclose(fpin_ptr);
   */


   /*   fprintf(stderr,"g:%lf, a0:%lf, a2:%lf, b12:%lf, b21:%lf, m1:%lf, c1:%lf, c2:%lf,\ntau:%lf, a1a:%lf, a1b:%lf, y10:%lf y20:%lf,]\n",best[0],best[1],best[2],best[3],best[4],best[5],best[6],best[7],best[8],best[9],best[10],best[11],best[12]);*/
   /*   fprintf(stderr,"nfeval = %i\n",nfeval);*/

   
  for(i=0;i<(NPARAMS+2*cc);i++){
    best2[i] = best3[i];
  }

  fp = fopen("fitted_params8a2c2b2.dat","w");
  for(i=0;i<P;i++){
    fprintf(fp,"%g\n",best2[i]);
  }
  fprintf(fp,"%%cost_function2:%f\n",cost_function2(data,cc2,D,best2,&nfeval,sim));
  fclose(fp);

  fp = fopen("fitted_patient.dat","w");

  for(i=0;i<cc2;i++){
    fprintf(fp,"%i:%lf %lf %lf %lf %lf\n",i,sim[i][0],sim[i][1],sim[i][2],sim[i][3],sim[i][4]);
  }

  fclose(fp);

  fprintf(stderr,"cost_function2:%f\n",cost_function2(data,cc2,D,best,&nfeval,sim));


  /*  best2[0] = 0.9472;
  best2[1] = 0.9817;
  best2[2] = 0.8001;
  best2[3] = 1.0035;
  best2[4] = 0.0955;
  best2[5] = 0.0005;
  best2[6] = 0.0013;
  best2[7] = 0.0000;
  best2[8] = 0.1939;
  best2[9] = 9.5486;
  best2[10] = 19.2422;
  best2[11] = 0.0182;
  best2[12] = 0.8585;
  best2[13] = 1.0100;
  best2[14] = 1.0192;
  best2[15] = 0.8137;
  best2[16] = 0.0671;
  best2[17] = 0.0138;
  best2[18] = 0.0075;
  best2[19] = 0.0240;
  best2[20] = 7.6523;
  best2[21] = 0.1954;

  fprintf(stderr,"before:\n");
  fprintf(stderr,"cost_function2:%f\n",cost_function2(data,cc2,D,best2,&nfeval,sim));*/


}
